import * as React from "react";
import { Fragment, jsx, jsxs } from "react/jsx-runtime";
import { useCallback, useContext, useMemo, useState } from "react";
import PropTypes from "prop-types";
import { styled, ThemeContext } from "@elliemae/ds-system";
import { DSPopover, PopperPositions as positions } from "@elliemae/ds-popover";
import { useCancellableDelayedCallback } from "./useCancellableDelayedCallback.js";
const dsTestId = "DS-SimpleTruncateText";
const isEllipsisActive = ({ offsetWidth, scrollWidth }) => offsetWidth < scrollWidth;
const initialTooltipState = (value = "", options = {}) => ({
  reference: null,
  visible: false,
  value,
  options
});
const Text = styled.span`
  text-overflow: ellipsis;
  white-space: nowrap;
  overflow: hidden;
  display: inline-block;
  max-width: 100%;
`;
const SimpleTruncatedTooltipText = (props) => {
  const { containerProps, tooltipDelay, placement, value, zIndex, tooltipOptions, textOptions } = props;
  const [tooltipState, setTooltipState] = useState(initialTooltipState(value, tooltipOptions));
  const show = useCallback(
    (newState) => {
      setTooltipState({ ...tooltipState, ...newState, visible: true });
    },
    [tooltipState]
  );
  const hideTooltip = useCallback(
    (newState) => {
      setTooltipState({ ...tooltipState, ...newState, visible: false });
    },
    [tooltipState]
  );
  const [showTooltip, cancelShowTooltip] = useCancellableDelayedCallback(show, tooltipDelay);
  const handleMouseEnter = useCallback(
    (e) => {
      const { target } = e;
      const SimpleTruncatedTextEl = target.closest(`[data-testid="${dsTestId}"]`);
      if (SimpleTruncatedTextEl && isEllipsisActive(SimpleTruncatedTextEl)) {
        showTooltip({ value, reference: SimpleTruncatedTextEl });
      }
    },
    [showTooltip, value]
  );
  const handleMouseLeave = useCallback(() => {
    cancelShowTooltip();
    hideTooltip({ reference: null });
  }, [hideTooltip, cancelShowTooltip]);
  const handlers = useMemo(() => {
    if (!showTooltip) return {};
    return {
      onMouseEnter: handleMouseEnter,
      onMouseLeave: handleMouseLeave
    };
  }, [showTooltip, handleMouseEnter, handleMouseLeave]);
  const theme = useContext(ThemeContext);
  const PurePopover = useMemo(
    () => /* @__PURE__ */ jsx(Fragment, { children: tooltipState.visible ? /* @__PURE__ */ jsx(
      DSPopover,
      {
        boundaries: "window",
        style: { pointerEvents: "none", zIndex: zIndex ?? theme.zIndex.tooltip },
        placement,
        content: tooltipState.value,
        referenceEl: tooltipState.reference,
        visible: tooltipState.visible,
        showArrow: true
      }
    ) : null }),
    [tooltipState, placement, zIndex]
  );
  const PureText = useMemo(
    () => /* @__PURE__ */ jsx(Fragment, { children: /* @__PURE__ */ jsx(
      Text,
      {
        ...containerProps && { ...containerProps },
        ...textOptions && { ...textOptions },
        ...handlers && { ...handlers },
        "data-testid": dsTestId,
        children: value
      }
    ) }),
    [containerProps, textOptions, handlers, value]
  );
  const PureSimpleTruncatedTooltipText = useMemo(
    () => /* @__PURE__ */ jsxs(Fragment, { children: [
      PureText,
      PurePopover
    ] }),
    [PureText, PurePopover]
  );
  return PureSimpleTruncatedTooltipText;
};
SimpleTruncatedTooltipText.propTypes = {
  containerProps: PropTypes.object,
  tooltipOptions: PropTypes.object,
  textOptions: PropTypes.object,
  /** Text that when truncated will trigger the tooltip interaction */
  value: PropTypes.oneOfType([PropTypes.string, PropTypes.number, PropTypes.node]),
  /** Position of the tooltip */
  placement: PropTypes.oneOf([
    positions.AUTO_START,
    positions.AUTO_END,
    positions.AUTO,
    positions.TOP_START,
    positions.TOP,
    positions.TOP_END,
    positions.RIGHT_START,
    positions.RIGHT,
    positions.RIGHT_END,
    positions.BOTTOM_START,
    positions.BOTTOM,
    positions.BOTTOM_END,
    positions.LEFT_START,
    positions.LEFT,
    positions.LEFT_END
  ]),
  /** Delay to show the tooltip */
  tooltipDelay: PropTypes.number,
  /** override default zIndex */
  zIndex: PropTypes.number
};
SimpleTruncatedTooltipText.defaultProps = {
  containerProps: {},
  tooltipOptions: {},
  textOptions: {},
  value: "",
  placement: positions.TOP,
  tooltipDelay: 200
};
var SimpleTruncatedTooltipText_default = SimpleTruncatedTooltipText;
export {
  SimpleTruncatedTooltipText,
  SimpleTruncatedTooltipText_default as default
};
//# sourceMappingURL=SimpleTruncatedTooltipText.js.map
